/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.usefulrp.rpprovider;

import java.util.ArrayList;
import java.util.Iterator;
import java.io.FileInputStream;
import java.lang.reflect.Constructor;
import javax.xml.namespace.QName;

import org.xml.sax.InputSource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axis.MessageContext;

import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.utils.Resources;
import org.globus.util.I18n;

import org.globus.wsrf.impl.BaseResourceProperty;
import org.globus.wsrf.config.ContainerConfig;
import org.globus.wsrf.config.ConfigException;
import org.globus.wsrf.utils.XmlUtils;
import org.globus.wsrf.utils.ContextUtils;

import org.globus.wsrf.encoding.ObjectDeserializer;

import org.globus.mds.usefulrp.types.ResourcePropertyProviderConfig;
import org.globus.mds.usefulrp.types.ResourcePropertyProviderConfigArray;

/**
 *  An GT4 OperationProvider for managing dynamic Resource Properties created with 
 *  the usefulrp Resource Property Provider API.  This class can be added (at 
 *  configuration time) as an operation provider to any GT4 Java WS-Core 
 *  compatible web service or resource that implements the
 *  {@link org.globus.wsrf.ResourceProperties org.globus.wsrf.ResourceProperties} 
 *  interface.<p>  
 *
 *  This operation provider is self-contained and reads all of its configuration 
 *  from a file specified by the <code>rpProviderConfigFile</code> name-value 
 *  parameter which is expected to be located in the hosting service's 
 *  <code>server-config.wsdd</code> service descriptor.
*/

public class ResourcePropertyProviderCollection
{
    private static Log logger =
            LogFactory.getLog(ResourcePropertyProviderCollection.class.getName());

    private static I18n i18n = I18n.getI18n(Resources.class.getName());
    
    public static final String CONFIG_FILE = "rpProviderConfigFile";
    private String configFile;
    private ResourcePropertySet rpSet;
    
    private boolean initialized = false;
    private ArrayList collection = new ArrayList();
    
   /**
    *  
    */
    public ResourcePropertyProviderCollection() 
    {
        MessageContext ctx = MessageContext.getCurrentContext();

        this.configFile = ContainerConfig.getBaseDirectory() + "/" +
            (String)ContextUtils.getServiceProperty(ctx, CONFIG_FILE);  
        
        Object resource = null;
        try {
            resource = ResourceContext.getResourceContext().getResource();
        } catch (Exception e) {
            logger.error(i18n.getMessage("resourceDisoveryFailed"), e);
            return;
        }

        if (!(resource instanceof ResourceProperties)) {
            logger.error(i18n.getMessage("rpsNotSupported"));
            return;
        }

        this.rpSet =
            ((ResourceProperties)resource).getResourcePropertySet();
        
        try {
            this.initialize();
        } catch (Exception e) {
            logger.warn("Could not initialize ResourcePropertyProviderCollection:" + e);
        }
        
    }
    
    public ResourcePropertyProviderCollection(String configFile,
                                              ResourcePropertySet rpSet,
                                              boolean initialize) 
    {
        this.configFile = configFile;
        this.rpSet = rpSet;
        
        if (!initialize) {
            return;
        }
        
        try {
            this.initialize();
        } catch (Exception e) {
            logger.warn("Could not initialize ResourcePropertyProviderCollection:" + e);
        }        
    }
    
    public void initialize() 
        throws Exception 
    {   
        if (!processConfig()) {
            return;
        }
        
        ResourceProperty rp = null;
        String rpImpl = null;
        
	Iterator iter = this.collection.iterator();
        while (iter.hasNext()) 
        {
       	    ResourcePropertyProviderConfig config = 
                (ResourcePropertyProviderConfig) iter.next();
            
            try 
            {
                rpImpl = config.getResourcePropertyImpl();
                
                if (rpImpl == null) {
                    throw new IllegalArgumentException(
                        "ResourceProperty implementation class name cannot be null!");    
                }
                
                Class cls = Class.forName(rpImpl.trim());                
                if (BaseResourcePropertyProvider.class.isAssignableFrom(cls)) 
                {
                    Constructor con = 
                        cls.getConstructor(
                            new Class[] {ResourcePropertyProviderConfig.class});
                        
                    rp = (ResourceProperty) con.newInstance(
                        new Object[] { config } );                    
                }
                else if (BaseResourceProperty.class.isAssignableFrom(cls)) 
                {
                    Constructor con = 
                        cls.getConstructor( new Class[] {QName.class} );
                        
                    rp = (ResourceProperty) con.newInstance(
                        new Object[] {config.getResourcePropertyName()} );                      
                }
                else if (ResourceProperty.class.isAssignableFrom(cls)) 
                {
                    rp = (ResourceProperty) cls.newInstance();
                }
                else
                {
                    throw new Exception("Unsupported RP implementation class " + 
                                        rpImpl +
                                        " does not implement any of the required interfaces");
                }
                
                this.rpSet.add(rp);
            }
            catch (Exception e)
            {
                logger.error(
                    "Exception during ResourceProperty class instantiation: " + e);
                logger.warn(
                    "The resource property " + 
                    config.getResourcePropertyName().toString() + 
                    " will not be available");
                continue;
            }
        }
    }
    
    private boolean processConfig()
    {
        boolean validConfig = false;

        try 
        { 
            ResourcePropertyProviderConfigArray config = 
                (ResourcePropertyProviderConfigArray)

                    ObjectDeserializer.deserialize(
                        new InputSource(
                            new FileInputStream(this.configFile)), 
                                ResourcePropertyProviderConfigArray.class);
                        
            ResourcePropertyProviderConfig[] array = config.getConfigArray();
            if ((array == null) || (array.length < 1)) 
            {
                throw new Exception("No valid configuration elements found");
            }
            
            for (int i=0;i<array.length;i++) {
                collection.add(array[i]);
            }
            
            validConfig = true;
            
        }
        catch(Exception e) 
        {
            logger.error("Exception while processing configuration file, some Resource Properties may not be available: ", e);
        }

        return validConfig;
    }
    
}


