/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.usefulrp.rpprovider;

import org.globus.mds.usefulrp.types.ResourcePropertyElementProducerConfig;

import commonj.timers.Timer;
import commonj.timers.TimerListener;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *  A <code>TimerListener</code> implementation for executing timer tasks that 
 *  periodically populate the values of a <code>BaseResourcePropertyProvider</code> 
 *  derived class.  This class performs the following actions when the timer 
 *  event is fired:<p>
 *
 *  1. Invokes the {@link ResourcePropertyElementProducer#getElement
 *     ResourcePropertyElementProducer.getElement} method of the 
 *     <code>ResourcePropertyElementProducer</code>
 *     class specified in the <code>ResourcePropertyElementProducerConfig</code> 
 *     parameter.<p>
 *
 *  2. Invokes the {@link ResourcePropertyProviderListener#executionPerformed 
 *     ResourcePropertyProviderListener.executionPerformed} method of the 
 *     instance of the Java class specified by the 
 *     <code>ResourcePropertyProviderListener</code> parameter, passing the 
 *     result of the previous invocation of the 
 *     <code>ResourcePropertyElementProducer.getElement()</code> method to the 
 *     appropriate <code>ResourcePropertyProviderListener</code> class.<p>
 *
 *  <b>Note:</b> If any errors or exceptions occur during execution, the default 
 *  behavior of this timer task will be to cancel itself.  This behavior can be 
 *  overridden by setting the <code>cancelOnErrors</code> parameter to false at 
 *  object construction time.
 */
public class ResourcePropertyProviderTask implements TimerListener
{
    private ResourcePropertyElementProducerConfig config;
    private ResourcePropertyProviderListener listener;
    private ResourcePropertyElementProducer producer;
    private Timer timer;
    private boolean cancel = true;
    private String[] trimmedArgs = null;    
    
    private static Log logger =
        LogFactory.getLog(ResourcePropertyProviderTask.class.getName());
        
        
    /** Creates a new instance of ResourcePropertyProviderTask 
     *  
     *  @param config an instance of 
     *      <code>ResourcePropertyElementProducerConfig</code>  
     *      that contains the configuration information for this task.
     *
     *  @param listener an instance of 
     *      <code>ResourcePropertyProviderListener</code> that will process the
     *      results from the execution of the <code>ResourcePropertyElementProducer</code>.     
     */
    public ResourcePropertyProviderTask(ResourcePropertyElementProducerConfig config,
                                        ResourcePropertyProviderListener listener) 
        throws Exception
    {  
        this.config = config;
        this.listener = listener; 
        
        this.initialize();
    }
    
    /** Creates a new instance of ResourcePropertyProviderTask 
     *  
     *  @param config an instance of 
     *      <code>ResourcePropertyElementProducerConfig</code>  
     *      that contains the configuration information for this task.
     *
     *  @param listener an instance of 
     *      <code>ResourcePropertyProviderListener</code> that will process the
     *      results from the execution of the <code>ResourcePropertyElementProducer</code>.
     *
     *  @param cancelOnErrors a boolean value indicating that the task should 
     *          automatically be cancelled if any processing errors occur.
     *          Defaults to true.   
     */
    public ResourcePropertyProviderTask(ResourcePropertyElementProducerConfig config,
                                        ResourcePropertyProviderListener listener,
                                        boolean cancelOnErrors) 
        throws Exception
    {       
        this.config = config;
        this.listener = listener; 
        this.cancel = cancelOnErrors;
        
        this.initialize();
    }

    protected void initialize() throws Exception
    {
        if (this.config == null) {
            throw new IllegalArgumentException("ResourcePropertyElementProducerConfig cannot be null!");
        }
        
        if (this.listener == null) {
            throw new IllegalArgumentException("ResourcePropertyProviderListener cannot be null!");
        }        
        
        String className = config.getClassName();
        if (className == null) {
            throw new IllegalArgumentException("ResourcePropertyElementProducer class name cannot be null!");
        }
        
        Class cls = Class.forName(className.trim());
        if (ResourcePropertyElementProducer.class.isAssignableFrom(cls)) {
            this.producer = (ResourcePropertyElementProducer) cls.newInstance();
        }
        
        // trim all input arguments
        String[] args = this.config.getArguments();
        if ((args != null) && (args.length > 0)) 
        {
            trimmedArgs = new String[args.length];
            for (int i=0;i<args.length;i++) {
                trimmedArgs[i] = args[i].trim();
            }               
        }          
    }
    
    public void setTimer(Timer t) {
        this.timer = t;
    }

    public Timer getTimer() {
        return this.timer;
    }
        
    public void timerExpired(Timer timer) 
    {
        Element result = null;
        
        try
        {
            result = this.producer.getElement(this.trimmedArgs);
            this.listener.executionPerformed(result);
        }
        catch (Exception e)
        {
            logger.error("Unhandled exception during execution of "  + 
                         config.getClassName() + ": " + e);
            
            if (this.cancel) 
            {
                logger.warn(
                    "Timer event canceled due to unhandled exception during execution of " + 
                    config.getClassName()
                );
                
                this.timer.cancel();
                return;
            }
        }        
    }
    
}
