/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.mds.usefulrp.rpprovider;

import java.util.ArrayList;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;

import org.w3c.dom.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.message.MessageElement;

import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.impl.timer.TimerManagerImpl;

import org.globus.mds.usefulrp.types.ResourcePropertyProviderConfig;
import org.globus.mds.usefulrp.types.ResourcePropertyElementProducerConfig;

/** 
 *   Generic class for using arbitrary <code>ResourcePropertyElementProducer</code>
 *   plugins to generate a single Resource Property value for a configuration-named 
 *   dynamic Resource Property. <p>
 *
 *   The <code>SingleValueResourcePropertyProvider</code> class is suitable for use in 
 *   situations where one or more <code>ResourcePropertyElementProducers</code> 
 *   are configured to generate a single data value for the named Resource 
 *   Property and when complex processing or merging of output fragments 
 *   (or a more specalized implmentation) is not necessary.
 *
 *   The intended usage of this class is in scenarios where the developer has
 *   existing XML data that can be easily ingested with one of the pre-existing
 *   <code>ResourcePropertyElementProducer</code> classes. <p>
 *
 *   Using the <code>ResourcePropertyProviderConfig</code> object, it is 
 *   possible to configure <code>SingleValueResourcePropertyProvider</code> to 
 *   import well-formed XML data from a variety of  input sources and represent 
 *   that data as a single ResourceProperty value.  
 *   The <code>SingleValueResourcePropertyProvider</code> class provides a reusable 
 *   mechanism to import such data without the developer having to write a 
 *   specialized <code>ResourceProperty</code> implementation of his/her own.<p>
 *
 *   The following simplifying assumptions are made about the 
 *   <code>SingleValueResourcePropertyProvider</code> data structure: <p>
 *
 *   1.  The cardinal order of the child elements in the single result value of 
 *       the Resource Property will match that of the cardinal order of the 
 *       <code>ResourcePropertyElementProducer</code> configuration blocks found 
 *       in the <code>ResourcePropertyProviderConfig</code> parameter.  
 *       In other words, the RP value [0] will be an XML element that contains
 *       a set of child elements which correspond (by ordinal position) the result
 *       output from the execution of each <code>ResourcePropertyElementProducer</code> 
 *       returned from 
 *       <code>ResourcePropertyProviderConfig.getResourcePropertyElementProducers()</code>,
 *       and so on.<p>
 *
 *       <b>Note:</b> It is important to keep the ordering restriction in mind 
 *       when enumerating the <code>ResourcePropertyElementProducers</code> in 
 *       the <code>ResourcePropertyProviderConfig</code> configuration file, 
 *       as the ordering of the child elements of the result RP value 
 *       will mirror that which is found in the file.<p>
 *
 *   2.  The XML output for any configured producer should have as its root 
 *       element an element with a QName that matches the QName of the Resource 
 *       Property as configured in the 
 *       <code>ResourcePropertyProviderConfig.resourcePropertyName</code> parameter.<p>
 *  
 *       <b>Note:</b> If the XML output of the producer does NOT contain such a 
 *       root, a new root will be created that matches the RP QName and the 
 *       XML output from the <code>ResourcePropertyElementProducer</code> will 
 *       be "wrapped" as a child element of this new root element. <p>
 *   
*/

public class SingleValueResourcePropertyProvider 
    extends BaseResourcePropertyProvider 
{
    private static Log logger =
            LogFactory.getLog(SingleValueResourcePropertyProvider.class.getName());
       
    protected TransformElementListener[] listeners;

    public SingleValueResourcePropertyProvider(ResourcePropertyProviderConfig config) 
        throws Exception
    {
        super(config);        
    }    

    public void initialize() throws Exception 
    {
        super.initialize();     
        this.getTimerManager().setPoolSize(2);
        this.listeners = 
            TransformElementListener.scheduleListeners(this.getConfig(), 
                this.getTimerManager());
    }
    
    public Object getDynamicValue() 
    {
        QName name = this.getConfig().getResourcePropertyName();
        Object obj = null;
        
        for (int i = 0; i < listeners.length;i++) 
        {
            obj = listeners[i].getObject();
            if ((obj != null) && (obj instanceof Element)) 
            {
                if ((name != null) && 
                    (!(name.getLocalPart().equals(
                      ((Element)obj).getLocalName()) &&
                       name.getNamespaceURI().equals
                       (((Element)obj).getNamespaceURI())))
                    ) 
                {
                    if (logger.isDebugEnabled()) {
                        logger.debug("RP value name does not match RP name " + 
                                     name.toString() + 
                                     " creating wrapper element for: " + 
                                     new QName(((Element)obj).getLocalName(),
                                                ((Element)obj).getNamespaceURI())
                                     );
                    }
                    MessageElement elem = new MessageElement(name);
                    try {
                        elem.setObjectValue(obj);
                    } catch (SOAPException e) {
                        logger.warn("Exception forming wrapper element: " + e);
                    }
                    obj = elem;
                }                                                           
            }
        }
        return obj;
    }           

}
