/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.usefulrp.rpprovider;

import java.util.ArrayList;

import javax.xml.namespace.QName;
import org.w3c.dom.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.impl.timer.TimerManagerImpl;

import org.globus.mds.usefulrp.types.ResourcePropertyProviderConfig;
import org.globus.mds.usefulrp.types.ResourcePropertyElementProducerConfig;

/** 
    <code>TransformElementListener</code> is a simple 
    <code>ResourcePropertyProviderListener</code> class
    that automatically invokes a transform class, if that class can be found 
    and loaded successfully at object construction time.
*/

public class TransformElementListener implements ResourcePropertyProviderListener
{
    private static Log logger =
        LogFactory.getLog(TransformElementListener.class.getName());
      
    private Object object = null;
    private ResourcePropertyElementTransform transform = null;
    private String[] transformArgs = null;

    public TransformElementListener(String transformClass, String[] transformArgs) 
    {
        if (transformClass == null) {
            return;
        }

        try 
        {
            Class cls = Class.forName(transformClass.trim());
            this.transform = 
                (ResourcePropertyElementTransform) cls.newInstance();
        }
        catch (Exception e) 
        {
            logger.warn("Unable to create instance of transform class " + 
                        transformClass + ": " + e);
        }
        
        // trim all arguments
        String[] trimmedArgs = null;
        if ((transformArgs != null) && (transformArgs.length > 0)) 
        {
            trimmedArgs = new String[transformArgs.length];
            
            for (int i=0;i<transformArgs.length;i++) {
                trimmedArgs[i] = transformArgs[i].trim();
            }            
        }
        this.transformArgs = trimmedArgs; 
        
    }

    public Object getObject() {
        return this.object;
    }

    public void executionPerformed(Element element) throws Exception 
    {
        if (transform != null) {
            this.object = 
                transform.transformElement(element, this.transformArgs);
        } else {
            this.object = element;
        }
    }
    
    /** 
     *  A convenience method for scheduling timer tasks for the 
     *  ResourcePropertyElementProducers contained in the 
     *  <code>ResourcePropertyProviderConfig</code> parameter into a 
     *  given TimerManagerImpl object.
     *
     *  @param config The <code>ResourcePropertyProviderConfig</code> 
     *      configuration information that contains the parameters needed 
     *      for provider execution 
     *
     *  @param timerManager An instance of <code>TimerManagerImpl</code> that 
     *      will manage the periodic execution of the 
     *      <code>ResourcePropertyElementProducers</code>
    */
    public static TransformElementListener[] scheduleListeners(
        ResourcePropertyProviderConfig config,
        TimerManagerImpl timerManager) 
    {
       
        ResourcePropertyElementProducerConfig[] producers = 
            config.getResourcePropertyElementProducers();
        if ((producers == null) || 
            ((producers != null) && (producers.length == 0))) {
            logger.warn("No ResourcePropertyElementProducers specified in configuration");
            return null;
        }
        
        TransformElementListener[] listeners = 
            new TransformElementListener[producers.length];
        
        for (int i=0;i<producers.length;i++) 
        {
            int period = 0;
            Integer integer = producers[i].getPeriod();
            if (integer != null) {
                period = integer.intValue() * 1000;
            }
            try 
            {
                ResourcePropertyProviderTask task = 
                    new ResourcePropertyProviderTask
                    (
                        producers[i],
                        listeners[i] = 
                            new TransformElementListener(
                                producers[i].getTransformClass(),
                                producers[i].getTransformArguments()
                            )                             
                    );
                
                if (period > 0) {
                    task.setTimer(timerManager.schedule(task, 0, period));
                } else {
                    task.setTimer(timerManager.schedule(task, 0));
                }
            }
            catch (Exception e) 
            {
                logger.warn("Exception while creating provider task for " + 
                            producers[i].getClassName() + 
                            "; the task will not be scheduled. Exception was: " + e);
                continue;
            }
        }
        return listeners;
    }
}
